﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;            //for file write/read
using System.Collections;   //for ArrayList
using Kst32bfont;           //CSF1 Font Format Decoder


// 2013.08.15 KaoruZenyouji (kaoruzen@gmail.com / @kaoruzen)


namespace CSF1toKiCADFontSrc
{
    public partial class MainForm : Form
    {
        public MainForm()
        {
            InitializeComponent();
        }

        //Initialize
        private void MainForm_Load(object sender, EventArgs e)
        {
            KstFontData = new Kst32bfont.Kst32bfont();

            if (!System.IO.File.Exists(Application.StartupPath + "\\KST32B.TXT"))
            {
                MessageBox.Show("Coudn't find KST32B font data. Copy \"KST32B.TXT\" to application directory.",
                    "Error", MessageBoxButtons.OK, MessageBoxIcon.Hand);

                Application.Exit();
            }
            KstFontData.FontPath = Application.StartupPath + "\\KST32B.TXT";
            KstFontData.LoadFont(); //load font data.

        }

        Kst32bfont.Kst32bfont KstFontData;


        /////////////////////////////////////////////////////////////
        //////////////  Compiled Font Viewer Function
        /////////////////////////////////////////////////////////////


        /// <summary>
        /// If textbox was clicked, delete all data.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void textBoxCompiledFont_MouseClick(object sender, MouseEventArgs e)
        {
            textBoxCompiledFont.Text = "";
        }

        private void textBoxCompiledFont_TextChanged(object sender, EventArgs e)
        {
            Bitmap image;
            try
            {
                image = renderCompiledFont(textBoxCompiledFont.Text);
            }
            catch
            {
                textBoxCompiledFont.BackColor = Color.Tomato;
                return;
            }

            textBoxCompiledFont.BackColor = Color.SkyBlue;
            pictureBoxViewer.Image = image;
        }

        /// <summary>
        /// Render compiled character data to bitmap.
        ///  example(Q_cap): "G]Z]X\\VZSWQVOV RP[NZLXKTKMLINGPFTFVGXIYMYTXXVZT[P["
        /// </summary>
        /// <param name="compiledCharData"></param>
        private Bitmap renderCompiledFont(string compiledCharData)
        {
            int scale = 2;   //

            compiledCharData = compiledCharData.Replace(" R", "##"); //replace " R" to "#" char tempolary
            compiledCharData = compiledCharData.Replace(" ", "");    //delete spaces
            compiledCharData = compiledCharData.Replace("\"", "");   //delete " char.
            compiledCharData = compiledCharData.Replace("\\\\", "\\");  //replace "\\" to "\"

            compiledCharData = compiledCharData.Substring(2, compiledCharData.Length - 2);

            if (compiledCharData.Length % 2 != 0)
            {
                throw new ArgumentException();
                // maybe Y coordinary losted
                //return;
            }

            compiledCharData = compiledCharData.Replace("##", "#"); //replace " R" to "#" char tempolary
                                                                    // (to use .Split method)
            string[] polygonData = compiledCharData.Split('#');

            //prepare graphics obj.
            Bitmap canvas = new Bitmap(pictureBoxViewer.Width, pictureBoxViewer.Height);
            Graphics g = Graphics.FromImage(canvas);
            Rectangle filled = new Rectangle(0, 0, canvas.Width, canvas.Height);
            g.FillRectangle(Brushes.Black, filled);

            double posX = 0.0;
            double posY = 0.0;

            // draw glyph
            for (int i = 0; i < polygonData.Length; i++)
            {
                double nxtPosX = 0.0;
                double nxtPosY = 0.0;

                for (int j = 0; j < polygonData[i].Length; j+=2)
                {
                    nxtPosX = ((int)(polygonData[i][j]) - 82) * scale + (canvas.Width / 2);
                    nxtPosY = ((int)(polygonData[i][j + 1]) - 91) * scale + (canvas.Height / 2);

                    if (j != 0)
                    {
                        // Draw Line
                        g.DrawLine(Pens.Aqua, 
                            (int)posX, (int)posY, (int)nxtPosX, (int)nxtPosY);
                    }

                    posX = nxtPosX;
                    posY = nxtPosY;
                }

            }

            //draw font width(??) arrow
            int x1 = (compiledCharData[0] - 82) * scale +(canvas.Width / 2);
            int x2 = (compiledCharData[1] - 82) * scale +(canvas.Width / 2);

            g.DrawLine(Pens.Red, x1 - 3, 1, x1 + 3, 1);
            g.DrawLine(Pens.Red, x1 - 3, 1, x1, 10);
            g.DrawLine(Pens.Red, x1 + 3, 1, x1, 10);

            g.DrawLine(Pens.Yellow, x2 - 3, 1, x2 + 3, 1);
            g.DrawLine(Pens.Yellow, x2 - 3, 1, x2, 10);
            g.DrawLine(Pens.Yellow, x2 + 3, 1, x2, 10);


            return canvas;
        }

        

        

        /////////////////////////////////////////////////////////////
        //////////////  KST32B to KiCad Compiled Font Converter Chk
        /////////////////////////////////////////////////////////////


        private void textBoxCheckChar_MouseClick(object sender, MouseEventArgs e)
        {
            textBoxCheckChar.Text = "";
        }

        
        //TODO "compileGlyph"メソッドと統合して整理すること！
        private void textBoxCheckChar_TextChanged(object sender, EventArgs e)
        {
            ArrayList kst32GlyphData;
            int width = 0;

            int scale = 2;

            int offsetY = 10;
            int offsetX = 0;

            string compiledFont = "";

            if (textBoxCheckChar.Text.Length < 1)
            {
                textBoxCheckChar.BackColor = Color.Tomato;
                return;
            }
            textBoxCheckChar.BackColor = Color.SkyBlue;

            //show unicode
            int code = (int)textBoxCheckChar.Text[0];
            labelUnicode.Text = "0x" + code.ToString("x4") + " (" + code.ToString() + ")";

            // get font line data
            kst32GlyphData = KstFontData.GetFontLineData(
                textBoxCheckChar.Text[0].ToString(),
                out width);

            //prepare graphics obj.
            Bitmap canvas = new Bitmap(pictureBoxCheck.Width, pictureBoxCheck.Height);
            Graphics g = Graphics.FromImage(canvas);
            Rectangle filled = new Rectangle(0, 0, canvas.Width, canvas.Height);
            g.FillRectangle(Brushes.Black, filled);

            // glyphsize????
            compiledFont = ((char)((-1 * (int)(width / 2)) + 82)).ToString();
            compiledFont = compiledFont + ((char)((+1 * (int)(width / 2)) + 82)).ToString();

            for (int i = 0; i < kst32GlyphData.Count; i++)
            {
                Kst32bfont.LineData line = new LineData();
                line = (LineData)kst32GlyphData[i];

                // convert KiCad coordinate
                int posX = (line.p0.X - width/2) + offsetX;
                int posY = (-1 * line.p0.Y) + offsetY;
                int nxtPosX = (line.p1.X - width/2) + offsetX;
                int nxtPosY = (-1 * line.p1.Y) + offsetY;

                // drawline
                g.DrawLine(Pens.Violet,
                            (int)posX * scale + canvas.Width / 2,
                            (int)posY * scale+ canvas.Height / 2 ,
                            (int)nxtPosX * scale+ canvas.Width / 2, 
                            (int)nxtPosY * scale+ canvas.Height / 2);

                //Compile Font
                compiledFont = compiledFont
                    + ((char)(posX + 82)).ToString()
                    + ((char)(posY + 91)).ToString()
                    + ((char)(nxtPosX + 82)).ToString()
                    + ((char)(nxtPosY + 91)).ToString();

                if (i + 1 != kst32GlyphData.Count)
                {
                    compiledFont = compiledFont + " R"; //sepalate all line.
                }
            }

            pictureBoxCheck.Image = canvas;

            compiledFont = compiledFont.Replace("\\", "\\\\");
            textBoxCompiledGlyph.Text = compiledFont;
        }

        



        /////////////////////////////////////////////////////////////
        //////////////  Font Merge
        /////////////////////////////////////////////////////////////


        private string compileGlyph(char data)
        {
            ArrayList kst32GlyphData;
            int width = 0;

            string compiledFont = "";

            int offsetY = 5;   //eeschemaでオリジナルのフォントと位置関係が合うように微調整
            int offsetX = 0;


            // get font line data
            kst32GlyphData = KstFontData.GetFontLineData(
                data.ToString(),
                out width);


            // glyphsize????
            compiledFont = ((char)((-1 * (int)(width / 2)) + 82)).ToString();
            compiledFont = compiledFont + ((char)((+1 * (int)(width / 2)) + 82)).ToString();

            for (int i = 0; i < kst32GlyphData.Count; i++)
            {
                Kst32bfont.LineData line = new LineData();
                line = (LineData)kst32GlyphData[i];

                // convert KiCad coordinate
                int posX = (line.p0.X - width / 2) + offsetX;
                int posY = (-1 * line.p0.Y) + offsetY;
                int nxtPosX = (line.p1.X - width / 2) + offsetX;
                int nxtPosY = (-1 * line.p1.Y) + offsetY;

                //Compile Font
                compiledFont = compiledFont
                    + ((char)(posX + 82)).ToString()
                    + ((char)(posY + 91)).ToString()
                    + ((char)(nxtPosX + 82)).ToString()
                    + ((char)(nxtPosY + 91)).ToString();

                if (i + 1 != kst32GlyphData.Count)
                {
                    compiledFont = compiledFont + " R"; //sepalate all line.
                }
            }


            compiledFont = compiledFont.Replace("\\", "\\\\");
            return compiledFont;
        }

        private void buttonSelectSrc_Click(object sender, EventArgs e)
        {
            OpenFileDialog openFileDiag = new OpenFileDialog();
            openFileDiag.FileName = "newstroke_font.cpp";
            openFileDiag.InitialDirectory = Application.StartupPath;
            openFileDiag.Filter ="C++ Source(*.cpp)|*.cpp|Other File(*.*)|*.*";

            // show dialog
            if (openFileDiag.ShowDialog() == DialogResult.OK)
            {
                textBoxSrcPath.Text = openFileDiag.FileName;
            }
        }

        private void buttonSelectDest_Click(object sender, EventArgs e)
        {
            SaveFileDialog saveFileDiag = new SaveFileDialog();
            saveFileDiag.FileName = "merged_arraydata.txt";
            saveFileDiag.InitialDirectory = Application.StartupPath;
            saveFileDiag.Filter = "text file(*.txt)|*.txt|Other File(*.*)|*.*";

            //show dialog
            if (saveFileDiag.ShowDialog() == DialogResult.OK)
            {
                textBoxDestPath.Text = saveFileDiag.FileName;
            }
        }

        private void buttonmerge_Click(object sender, EventArgs e)
        {
            if (!System.IO.File.Exists(textBoxSrcPath.Text))
            {
                MessageBox.Show("Couldn't find src file.", "Error", MessageBoxButtons.OK);
                return;
            }


            string[] fontTable = new string[65535];     //huge

            //iniitalize
            for (int i = 0; i < fontTable.Length; i++)
            {
                fontTable[i] = string.Empty;
            }

            //make font table from original
            using (StreamReader sr = new StreamReader(
                textBoxSrcPath.Text, Encoding.GetEncoding("Shift_JIS")))
            {
                string line = "";
                int unicode = 0x20;
                while ((line = sr.ReadLine()) != null)
                {
                    if(line.Contains("    \""))
                    {
                        if (!line.Contains("\"F^K[KFYFY[K[\","))
                        {
                            System.Text.RegularExpressions.Regex r =
                                new System.Text.RegularExpressions.Regex(
                                    "\"(.*?)\"",
                                    System.Text.RegularExpressions.RegexOptions.IgnoreCase);
                            System.Text.RegularExpressions.MatchCollection mc = r.Matches(line);

                            if(mc.Count > 0)
                            {   
                                fontTable[unicode] = mc[0].Groups[0].Value.Replace("\"", "");
                            }
                        }
                        unicode++;
                    }
                }
            }


            //merge fonts
            for (int i = 0x20; i < fontTable.Length; i++)
            {
                if (fontTable[i].Length == 0)   //font glyph unregistered
                {
                    if (KstFontData.getFont(KstFontData.ConvertCharCode((char)i)) != null)
                    {
                        fontTable[i] = compileGlyph((char)i);
                    }
                }
            }


            //output merged array src.
            using (StreamWriter dest = new StreamWriter(textBoxDestPath.Text))
            {
                for (int i = 0x20; i < fontTable.Length; i++) //0x20 = 32d
                {
                    if (i % 16 == 0)
                    {
                        dest.WriteLine("");
                        dest.WriteLine("/* 0x" + i.ToString("X4") + " - " + i.ToString() + " */");
                        dest.WriteLine("");
                    }

                    if (fontTable[i].Length == 0)
                    {
                        dest.WriteLine("   \"F^K[KFYFY[K[\",");
                    }
                    else
                    {
                        dest.WriteLine("   \"" + fontTable[i] + "\", ");
                    }
                }
            }
            
            //done.
            MessageBox.Show("Finished.", "Done.", MessageBoxButtons.OK);

        }


    }
}
